local flux = require "flux"

function love.load(arg)
  gameHeight = love.graphics.getHeight()
	gameWidth = love.graphics.getWidth()
	
	colors = {}
	colors[1] = {255, 0, 0}
	colors[2] = {0, 0, 255}
	colors[3] = {0, 255, 0}

	rows = 9
	cols = 9

	space = math.ceil(((1/math.max(rows, cols))*32)/2)*2

	height = (gameHeight / rows) - space
	width = (gameWidth / cols) - space

	board = {}

	for ri = 1, rows do
		board[ri] = {}
		for ci = 1, cols do
			local tx = (ci-1)*width+space*(ci-1)+space/2
			local ty = (ri-1)*height+space*(ri-1)+space/2
			local square = {
				x = tx, y = ty,
				w = width, h = height,
				c = love.math.random(1, 3), alpha = 255,
				tween = nil,
				tc = 0
			}
			board[ri][ci] = square
		end
	end
end

function tweenit(obj, vars)
	obj.tc = obj.tc + 1
	if obj.tween == nil then
		obj.tween = flux.to(obj, 0.2, vars)
			:ease("linear"):oncomplete(
				function()
					obj.tc = obj.tc - 1
					if obj.tc == 0 then
						obj.tween = nil
					end
				end)
	else
	obj.tween = obj.tween:after(obj, 0.2, vars)
		:ease("linear"):oncomplete(
				function()
					obj.tc = obj.tc - 1
					if obj.tc == 0 then
						obj.tween = nil
					end
				end)
	end
end

function love.update(dt)
	flux.update(dt)
end

function love.keyreleased(key)
	if key == "right" then
		for ri = 1, rows do
			for ci = 1, cols do
				local cur = board[ri][ci]
				for ti = ci+1, cols do
					if cur.c == board[ri][ti].c then
						for mi = ci+1, ti-1 do
							local tmp = board[ri][mi]
							--board[ri][mi].x = (mi-2)*width+space*(mi-2)+space/2
							tweenit(tmp, { x = (mi-2)*width+space*(mi-2)+space/2 })
						end
						table.insert(board[ri], ti-1, table.remove(board[ri], ci))
						--board[ri][ti-1].x = (ti-2)*width+space*(ti-2)+space/2
						tweenit(cur, { x = (ti-2)*width+space*(ti-2)+space/2 })
						break
					end
				end
			end
		end
	end
	if key == "left" then
		for ri = 1, rows do
			for ci = cols, 1, -1 do
				local cur = board[ri][ci]
				for ti = ci-1, 1, -1 do
					if cur.c == board[ri][ti].c then
						for mi = ci-1, ti+1, -1 do
							local tmp = board[ri][mi]
							--board[ri][mi].x = (mi)*width+space*(mi)+space/2
							tweenit(tmp, { x = (mi)*width+space*(mi)+space/2 })
						end
						table.insert(board[ri], ti+1, table.remove(board[ri], ci))
						--board[ri][ti+1].x = (ti)*width+space*(ti)+space/2
							tweenit(cur, { x = (ti)*width+space*(ti)+space/2 })
						break
					end
				end
			end
		end
	end
	if key == "down" then
		for ci = 1, cols do
			for ri = 1, rows do
				local cur = board[ri][ci]
				for ti = ri+1, rows do
					if cur.c == board[ti][ci].c then
						for mi = ri+1, ti-1 do
							local tmp = board[mi][ci]
							board[mi-1][ci] = tmp
							--board[mi-1][ci].y = (mi-2)*height+space*(mi-2)+space/2
							tweenit(tmp, { y = (mi-2)*height+space*(mi-2)+space/2 })
						end
						board[ti-1][ci] = cur
						--board[ti-1][ci].y = (ti-2)*height+space*(ti-2)+space/2
						tweenit(cur, { y = (ti-2)*height+space*(ti-2)+space/2 })
						break
					end
				end
			end
		end
	end
	if key == "up" then
		for ci = 1, cols do
			for ri = rows, 1, -1 do
				local cur = board[ri][ci]
				for ti = ri-1, 1, -1 do
					if cur.c == board[ti][ci].c then
						for mi = ri-1, ti+1, -1 do
							local tmp = board[mi][ci]
							board[mi+1][ci] = tmp
							--board[mi+1][ci].y = (mi)*height+space*(mi)+space/2
							tweenit(tmp, { y = (mi)*height+space*(mi)+space/2 })
						end
						board[ti+1][ci] = cur
						--board[ti+1][ci].y = (ti)*height+space*(ti)+space/2
						tweenit(cur, { y = (ti)*height+space*(ti)+space/2 })
						break
					end
				end	
			end
		end
	end
end

function love.draw()
	for ri = 1, rows do
		for ci = 1, cols do
			local b = board[ri][ci]
			love.graphics.setColor(colors[board[ri][ci].c]) 
			love.graphics.rectangle("fill", b.x, b.y, b.w, b.h)
			--[[
			love.graphics.setColor(255, 255, 255) 
			love.graphics.print("x = "..b.x.."\ny = "..b.y..
			"\nw = "..b.w.."\nh = "..b.h, b.x+10, b.y+10)
			]]
		end
	end
end


function print_r(t, name, indent)
  local tableList = {}
  function table_r (t, name, indent, full)
    local id = not full and name
        or type(name)~="number" and tostring(name) or '['..name..']'
    local tag = indent..id..' = '
    local out = {}	-- result
    if type(t) == "table" then
      if tableList[t] ~= nil then table.insert(out, tag..'{} -- '..tableList[t]..' (self reference)')
      else
        tableList[t]= full and (full..'.'..id) or id
        if next(t) then -- Table not empty
          table.insert(out, tag .. '{')
          for key,value in pairs(t) do 
            table.insert(out,table_r(value, key, indent..'|  ', tableList[t]))
          end 
          table.insert(out,indent .. '}')
        else table.insert(out,tag .. '{}') end
      end
    else 
      local val = type(t) ~= "number" and type(t) ~= "boolean" and '"'..tostring(t)..'"' or tostring(t)
      table.insert(out, tag..val)
    end
    return table.concat(out, '\n')
  end
  return table_r(t, name or 'Value', indent or '')
end
 
function pr(t, name)
  print(print_r(t, name))
end
